@@ -45,7 +45,8 @@ class GeodataBuild:
     """
 
     def __init__(self, directory: str, display_progress,
-                 show_message:bool, exit_on_error:bool, languages_list_dct:{}, feature_code_list_dct:{}, supported_countries_dct:{}):
+                 show_message:bool, exit_on_error:bool, languages_list_dct:{}, feature_code_list_dct:{}, 
+                 supported_countries_dct:{},volume=''):
         """
         Read in datafiles needed for geodata, filter them and create a sql db.
         Filter dictionary examples:   
@@ -66,7 +67,9 @@ class GeodataBuild:
         self.show_message = show_message
         self.geoid_main_dict = {}  # Key is GEOID, Value is DB ID for entry
         self.geoid_admin_dict = {}  # Key is GEOID, Value is DB ID for entry
-            
+        self.volume = volume
+        self.collate = 'COLLATE NOCASE'
+
         self.exit_on_error = exit_on_error
         self.required_db_version = 4
         # Message to user upgrading from earlier DB version  
@@ -81,10 +84,13 @@ class GeodataBuild:
         self.feature_code_list_dct = feature_code_list_dct
         self.supported_countries_dct = supported_countries_dct
         self.lang_list = []
+        self.norm = Normalize.Normalize()
 
         for item in self.languages_list_dct:
             self.lang_list.append(item)
-
+            
+        if volume != '':
+            os.chdir(volume)
         if not os.path.exists(sub_dir):
             self.logger.warning(f'Directory] {sub_dir} NOT FOUND')
             if self.show_message:
@@ -121,6 +127,9 @@ class GeodataBuild:
         if self.geodb is None:
             self.logger.error('Cannot create DB: geodb is None')
             return True
+        
+        if self.volume != '':
+            os.chdir(self.volume)
 
         self.create_tables()
         
@@ -170,10 +179,10 @@ class GeodataBuild:
             self.logger.info(f'Alternate names done.  Elapsed ={(time.time() - start_time):.0f} seconds')
             
         self.create_alt_indices()
-        self.logger.info(f'Geonames entries = {self.geodb.get_row_count():,}')
+        #self.logger.info(f'Geonames entries = {self.geodb.get_row_count():,}')
 
         # Add aliases
-        Normalize.add_aliases_to_db(self)
+        self.norm.add_aliases_to_db(self)
 
         # Done - Set Database Version
         self.insert_version(self.required_db_version)
@@ -199,7 +208,10 @@ class GeodataBuild:
                                  ' admin2_id admin3_id admin4_id pop elev dem timezone mod')
         self.line_num = 0
         self.progress("Reading {}...".format(file), 0)
-        path = os.path.join(self.directory, file)
+        if self.volume != '':
+            os.chdir(self.volume)
+        path = os.path.join(self.volume, self.directory, file)
+        self.logger.info(f'Path = [{path}]')
 
         if os.path.exists(path):
             fsize = os.path.getsize(path)
@@ -227,7 +239,7 @@ class GeodataBuild:
                     if geoname_row.iso.lower() in self.supported_countries_dct and \
                             geoname_row.feat_code in self.feature_code_list_dct:
                         self.insert_georow(geoname_row)
-                        if geoname_row.name.lower() != Normalize.normalize(geoname_row.name, remove_commas=True):
+                        if geoname_row.name.lower() != self.norm.normalize(geoname_row.name, remove_commas=True):
                             self.insert_alternate_name(geoname_row.name,
                                                              geoname_row.id, 'ut8')
 
@@ -315,10 +327,11 @@ class GeodataBuild:
 
         self.logger.debug(f'{err_msg}')
         if err_msg == '':
+            pass
             # No DB errors detected
-            count = self.geodb.get_row_count()
-            self.logger.info(f'Geoname database has {count:,} entries\n'
-                             f'------------------------------------------------------------\n')
+            #count = self.geodb.get_row_count()
+            #self.logger.info(f'Geoname database has {count:,} entries\n'
+            #                 f'------------------------------------------------------------\n')
         else:
             # DB error detected - rebuild database if flag set
             if self.show_message:
@@ -340,8 +353,7 @@ class GeodataBuild:
                 return self.create_geonames_database()
         return False
 
-    @staticmethod
-    def update_geo_row_name(geo_row:[], name:str, normalize=True):
+    def update_geo_row_name(self, geo_row:[], name:str, normalize=True):
         """
             Update the name entry and soundex entry with a new location name
         #Args:
@@ -349,7 +361,7 @@ class GeodataBuild:
             name: location name
         """
         if normalize:
-            geo_row[GeoSearch.Entry.NAME] = Normalize.normalize(name, remove_commas=True)
+            geo_row[GeoSearch.Entry.NAME] = self.norm.normalize(name, remove_commas=True)
         else:
             geo_row[GeoSearch.Entry.NAME] = name.lower()
 
@@ -377,7 +389,7 @@ class GeodataBuild:
         geo_row[GeoSearch.Entry.ID] = geoname_row.id
             
         # Simplify feature type for Abbey/Priory, Castle, and Church.  Set feature based on population
-        geo_row[GeoSearch.Entry.FEAT] = Normalize.feature_normalize(feature=geo_row[GeoSearch.Entry.FEAT],
+        geo_row[GeoSearch.Entry.FEAT] = self.norm.feature_normalize(feature=geo_row[GeoSearch.Entry.FEAT],
                                                                     name=geo_row[GeoSearch.Entry.NAME], pop=int(geoname_row.pop))
             
         self.insert(geo_tuple=geo_row, feat_code=geoname_row.feat_code)
@@ -432,14 +444,14 @@ class GeodataBuild:
         Create all the tables needed for the geoname database
         """
         # name, country, admin1_id, admin2_id, lat, lon, feature, geoid
-        sql_geodata_table = """CREATE TABLE IF NOT EXISTS geodata    (
+        sql_geodata_table = f"""CREATE TABLE IF NOT EXISTS geodata    (
                 id           integer primary key autoincrement not null,
-                name     text COLLATE NOCASE,
+                name     text {self.collate},
                 country     text COLLATE NOCASE,
                 admin1_id     text COLLATE NOCASE,
                 admin2_id text COLLATE NOCASE,
-                lat      text  ,
-                lon       text ,
+                lat      text  COLLATE NOCASE,
+                lon       text COLLATE NOCASE,
                 feature      text COLLATE NOCASE,
                 geoid      text COLLATE NOCASE,
                 sdx     text COLLATE NOCASE
@@ -452,8 +464,8 @@ class GeodataBuild:
                 country     text COLLATE NOCASE,
                 admin1_id     text COLLATE NOCASE,
                 admin2_id text COLLATE NOCASE,
-                lat      text  ,
-                lon       text ,
+                lat      text  COLLATE NOCASE,
+                lon       text COLLATE NOCASE,
                 feature      text COLLATE NOCASE,
                 geoid      text COLLATE NOCASE,
                 sdx     text COLLATE NOCASE
@@ -462,8 +474,8 @@ class GeodataBuild:
         # name, lang, geoid
         sql_alt_name_table = """CREATE TABLE IF NOT EXISTS altname    (
                 id           integer primary key autoincrement not null,
-                name     text,
-                lang     text,
+                name     text COLLATE NOCASE,
+                lang     text COLLATE NOCASE,
                 geoid      text COLLATE NOCASE,
                 sdx     text COLLATE NOCASE
                                     );"""
@@ -488,7 +500,7 @@ class GeodataBuild:
         #Returns: None   
 
         """
-        sdx = GeoSearch.get_soundex(Normalize.normalize(alternate_name, True))
+        sdx = GeoSearch.get_soundex(self.norm.normalize(alternate_name, True))
         row = (alternate_name, lang, geoid, sdx)
         sql = ''' INSERT OR IGNORE INTO altname(name,lang, geoid, sdx)
                   VALUES(?,?,?,?) '''
@@ -531,10 +543,10 @@ class GeodataBuild:
         """
         # Indices for geodata table
         self.geodb.db.create_index(create_index_sql='CREATE INDEX IF NOT EXISTS name_idx ON geodata(name  , country, admin1_id  )')
-        self.geodb.db.create_index(create_index_sql='CREATE INDEX IF NOT EXISTS admin1_idx ON geodata(admin1_id   )')
-        self.geodb.db.create_index(create_index_sql='CREATE INDEX IF NOT EXISTS sdx_idx ON geodata(sdx  , country   )')
-        self.geodb.db.create_index(create_index_sql='CREATE INDEX IF NOT EXISTS adm_admin2_idx ON geodata(admin1_id  , feature  , '
-                                                    'admin2_id  )')
+        self.geodb.db.create_index(create_index_sql='CREATE INDEX IF NOT EXISTS name_idx2 ON geodata(name  , country, feature  )')
+        self.geodb.db.create_index(create_index_sql='CREATE INDEX IF NOT EXISTS admin1_idx ON geodata(admin1_id , country  )')
+        self.geodb.db.create_index(create_index_sql='CREATE INDEX IF NOT EXISTS sdx_idx ON geodata(sdx  , country, feature   )')
+        self.geodb.db.create_index(create_index_sql='CREATE INDEX IF NOT EXISTS admin2_idx ON geodata(admin1_id  , feature  , admin2_id )')
 
         # Indices for admin table
         self.geodb.db.create_index(create_index_sql='CREATE INDEX IF NOT EXISTS adm_name_idx ON admin(name  , country  )')
