@@ -71,9 +71,10 @@ class Loc:
         self.feature: str = ''  # Geoname feature code
         self.place_type: int = PlaceType.COUNTRY  # Is this a Country , Admin1 ,admin2 or city?
         self.geoid: str = ''  # Geoname GEOID
-        self.enclosed_by = ''   # The entity that encloses this.  E.g United States encloses Texas
+        self.enclosed_by = ''  # The entity that encloses this.  E.g United States encloses Texas
         self.updated_entry = ''
         self.score = 100.0
+        self.norm = Normalize.Normalize()
 
         # Lookup result info
         self.status: str = ""
@@ -98,7 +99,7 @@ class Loc:
         self.admin2_id = ""  # Admin2 Geoname ID
         self.prefix: str = ""  # Prefix (entries before city)
         self.feature: str = ''  # Geoname feature code
-        self.place_type: int = PlaceType.COUNTRY  # Is this a Country , Admin1 ,admin2 or city?
+        self.place_type: int = PlaceType.COUNTRY  # Is this a country , admin1 ,admin2 or city?
         self.geoid: str = ''  # Geoname GEOID
         self.enclosed_by = ''
         self.updated_entry = ''
@@ -111,9 +112,9 @@ class Loc:
         self.result_type_text: str = ''  # Text version of result type
         self.georow_list: List[Tuple] = [()]  # List of items that matched this location
 
-        self.georow_list.clear()
+        #self.georow_list.clear()
 
-    def parse_place(self, place_name: str, geo_db:GeoDB.GeoDB):
+    def parse_place(self, place_name: str, geo_db: GeoDB.GeoDB):
         """
             Given a comma separated place name,   
             parse into its city, admin1, country and type of entity (city, country etc)   
@@ -152,16 +153,18 @@ class Loc:
             self.logger.debug('filter')
             self.get_filter_parameters(place_name)
             return
-        
+
         if token_count > 0:
             #  COUNTRY - right-most token should be country
-            self.country_name = Normalize.normalize(tokens[-1], False)
+            self.country_name = self.norm.normalize(tokens[-1], False)
 
             # Validate country
-            #self.logger.debug(f'1) Find COUNTRY [{self.country_name}] *******')
-            self.country_iso = geo_db.s.get_country_iso(self.georow_list, self.country_name, self)  # Get Country country_iso
+            self.country_iso = geo_db.s.get_country_iso(self.country_name)  # Get Country country_iso
+            self.logger.debug(f'1) Lookup COUNTRY [{self.country_name}] Found ISO [{self.country_iso}] *******')
+
             if self.country_iso != '':
                 self.place_type = PlaceType.COUNTRY
+                self.result_type = GeoUtil.Result.PARTIAL_MATCH
             else:
                 # Last token is not COUNTRY.
                 # Append dummy token  so we now have <tokens>, x
@@ -169,29 +172,34 @@ class Loc:
                 token_count = len(tokens)
                 self.result_type = GeoUtil.Result.NO_COUNTRY
                 self.country_name = ''
-            #self.logger.debug(f'ISO =[{self.country_iso}]')
+            # self.logger.debug(f'ISO =[{self.country_iso}]')
         if token_count > 1:
             #  See if 2nd to last token is Admin1
             val = tokens[-2]
-            #self.logger.debug(f'ADM1 tkn-2 [{val}]')
-            self.admin1_name = Normalize.admin1_normalize(val, self.country_iso)
+            self.logger.debug(f'Get ADM1 from tkn-2 [{val}]')
+            self.admin1_name = self.norm.admin1_normalize(val, self.country_iso)
 
             if len(self.admin1_name) > 0:
                 # Lookup Admin1
-                #self.logger.debug(f'2) Find ADMIN1 [{self.admin1_name}] *******')
+                self.logger.debug(f'2) Find ADMIN1 [{self.admin1_name}] *******')
                 row_list = []
-                geo_db.s.get_admin1_id(self.admin1_name, self, row_list)
+                self.admin1_id = geo_db.s.get_admin1_id(self.admin1_name, self.country_iso)
                 if self.admin1_id != '':
                     # Found Admin1
                     self.place_type = PlaceType.ADMIN1
                     self.georow_list = row_list
+                    
                     self.admin1_name = geo_db.s.get_admin1_name(self.admin1_id, self.country_iso)
-                    #self.logger.debug(f'adm1 nm={self.admin1_name}')
+                    # self.logger.debug(f'adm1 nm=[{self.admin1_name}]\nGet ISO')
+                    self.logger.debug(f'2) Find iso for admin1 id [{self.admin1_id}] *******')
+
+                    self.country_iso = geo_db.s.get_iso_from_admin1_id(self.admin1_id, self.country_iso)
+
                     self.result_type = GeoUtil.Result.PARTIAL_MATCH
                     # Get country if blank
                     row_list = []
                     if self.country_name == '':
-                        self.country_name = geo_db.s.get_country_name(self.country_iso, row_list) 
+                        self.country_name = geo_db.s.get_country_name(self.country_iso)
                 else:
                     # Last token is not Admin1 - append dummy token so we have <tokens>, admin1, country
                     self.admin1_name = ''
@@ -201,7 +209,7 @@ class Loc:
             else:
                 tokens[-2] = '_'
 
-        # Last two tokens are now Admin1, Country (although they may be '_')
+        # Last two tokens are now Admin1, Country (although they may have dummy value '_')
         # If >2 tokens:  Put first non-blank token in City and in Prefix
         # If >3 tokens:  Put second non-blank token in Admin2 and also append to Prefix
 
@@ -212,35 +220,35 @@ class Loc:
         if token_count >= 3:
             #  Possible Formats: City, Admin1, Country or  Admin2, Admin1, Country
             #  Take first tkn as city
-            self.city = Normalize.normalize(tokens[0], False)
+            self.city = self.norm.normalize(tokens[0], False)
             self.place_type = PlaceType.CITY
 
             # Also place token[0] into Prefix
             if '*' not in tokens[0]:
                 self.prefix = str(tokens[0].strip(' '))
-                
+
         if token_count >= 4:
             #  Admin2 is 2nd.  Note -  if Admin2 isnt found, it will look it up as city
-            
+
             if GeoUtil.is_street(tokens[-4].lower()):
                 #  Format: Prefix, City, Admin1, Country
-                self.city = Normalize.normalize(tokens[-3], False)
+                self.city = self.norm.normalize(tokens[-3], False)
             else:
                 #  Format: City, Admin2, Admin1, Country
-                self.admin2_name = Normalize.normalize(tokens[-3], False)
-                self.city = Normalize.normalize(tokens[-4], False)
-                
+                self.admin2_name = self.norm.normalize(tokens[-3], False)
+                self.city = self.norm.normalize(tokens[-4], False)
+
             self.place_type = PlaceType.CITY
 
             # put token[0] and  token[1] into Prefix
             if '*' not in tokens[1]:
                 self.prefix = str(tokens[0].strip(' ')) + ' ' + str(tokens[1].strip(' '))
 
-        self.prefix = Normalize.normalize(self.prefix, False)
+        self.prefix = self.norm.normalize(self.prefix, False)
         row_list = []
         # fill in country name if still missing - finding Admin1 will find country ISO
         if self.country_name == '' and self.country_iso != '':
-            self.country_name = geo_db.s.get_country_name(self.country_iso, row_list)
+            self.country_name = geo_db.s.get_country_name(self.country_iso)
 
         self.logger.debug(f"    ======= PARSED: {place_name} \nCity [{self.city}] Adm2 [{self.admin2_name}]"
                           f" Adm1 [{self.admin1_name}] adm1_id [{self.admin1_id}] Cntry [{self.country_name}] Pref=[{self.prefix}]"
@@ -268,11 +276,11 @@ class Loc:
         parser.add_argument("-f", "--feature", help=argparse.SUPPRESS)
         parser.add_argument("-i", "--iso", help=argparse.SUPPRESS)
         parser.add_argument("-c", "--country", help=argparse.SUPPRESS)
-        self.logger.debug(f'Args {args}' )
+        self.logger.debug(f'Args {args}')
 
         try:
             options = parser.parse_args(args)
-            self.city = Normalize.normalize(tokens[0], False)
+            self.city = self.norm.normalize(tokens[0], False)
             self.place_type = PlaceType.ADVANCED_SEARCH
             self.logger.debug(options)
 
@@ -286,7 +294,7 @@ class Loc:
                 self.logger.debug(f'ft {self.feature}')
         except Exception as e:
             self.logger.debug(e)
-            
+
         self.logger.debug(f"    ======= PARSE ADV: {place_name} \nCity [{self.city}] Adm2 [{self.admin2_name}]"
                           f" Adm1 [{self.admin1_name}] adm1_id [{self.admin1_id}] Cntry [{self.country_name}] Pref=[{self.prefix}]"
                           f" type_id={self.place_type} iso={self.country_iso} feat={self.feature}")
@@ -304,7 +312,7 @@ class Loc:
 
     def add_commas(self, txt) -> str:
         if txt == '':
-                return ''
+            return ''
         else:
             return f'{txt}, '
 
@@ -334,10 +342,13 @@ class Loc:
         #Returns:  
             long name  
         """
+
+        # Put in appropriate commas
         city = self.add_commas(self.city)
         admin2 = self.add_commas(self.admin2_name)
         admin1 = self.add_commas(self.admin1_name)
 
+        # Format name based on place type
         if self.place_type == PlaceType.COUNTRY:
             nm = f"{self.country_name}"
         elif self.place_type == PlaceType.ADMIN1:
@@ -347,8 +358,8 @@ class Loc:
         else:
             nm = f"{city}{admin2}{admin1}{str(self.country_name)}"
 
-        # normalize prefix
-        self.prefix = Normalize.normalize(self.prefix, False)
+        # Normalize prefix
+        self.prefix = self.norm.normalize(self.prefix, False)
 
         if len(self.prefix) > 0:
             self.prefix_commas = ', '
@@ -363,7 +374,7 @@ class Loc:
                 nm = re.sub(key, replace_dct[key], nm)
 
         return nm
-    
+
     def get_display_name(self, replace_dct) -> str:
         """
         Take the fields in a Place and build full name.  See if alternate name is available
@@ -374,10 +385,10 @@ class Loc:
         #Returns:  
             long name  
         """
-        
+
         # See if there is an alternate name for this
-        #city, lang = self.geo_db.s.get_alternate_name(self.geoid)
-        #if city == '':
+        # city, lang = self.geo_db.s.get_alternate_name(self.geoid)
+        # if city == '':
         city = self.city
         city = self.add_commas(city)
         admin2 = self.add_commas(self.admin2_name)
@@ -393,7 +404,7 @@ class Loc:
             nm = f"{city}{admin2}{admin1}{str(self.country_name)}"
 
         # normalize prefix
-        self.prefix = Normalize.normalize(self.prefix, False)
+        self.prefix = self.norm.normalize(self.prefix, False)
 
         if len(self.prefix) > 0:
             self.prefix_commas = ', '
@@ -414,7 +425,7 @@ class Loc:
         #     prefix,city,county,state,country  (prefix plus long name)
 
         # Normalize country name
-        country, modified = Normalize.country_normalize(self.country_name)
+        country, modified = self.norm.country_normalize(self.country_name)
         full_title = self.prefix + ' ,' + f"{self.city}, {self.admin2_name}, {self.admin1_name}, {str(country)}"
         return full_title
 
@@ -495,7 +506,7 @@ class Loc:
         return result_sdx
 
     @staticmethod
-    def matchscore_prefix(pref: str, result: str) -> str:
+    def prefix_cleanup(pref: str, result: str) -> str:
         """
         Remove any items from prefix that are in match result.  Remove *   
         #Args:   
@@ -546,7 +557,7 @@ class Loc:
         return res.strip(',')
 
     @staticmethod
-    def prefix_cleanup(pref: str, result: str) -> str:
+    def fast_prefix(pref: str, result: str) -> str:
         """
         Cleanup prefix.  Remove any items from prefix that are in match result.  Remove *   
         #Args:   
@@ -556,7 +567,9 @@ class Loc:
         #Returns:  Prefix with words removed
 
         """
-        return Loc.matchscore_prefix(pref, result)
+        return Loc.prefix_cleanup(pref, result)
+        pref, res = GeoUtil.remove_matching_sequences(text1=pref,text2=result,min_len=2)
+        return pref
 
     @staticmethod
     def get_district1_type(iso) -> str:
@@ -583,9 +596,11 @@ class Loc:
             nm = ''
         return nm
 
-    def update_names(self,  dct):
+    def update_names(self, dct):
+        self.logger.debug(f'pref bef=[{self.prefix}]')
         prfx = self.prefix_cleanup(self.prefix, self.get_long_name(dct))
         self.updated_entry = GeoUtil.capwords(prfx) + self.get_long_name(dct)
+        self.logger.debug(f'updated_entry=[{self.updated_entry}]')
 
 
 def remove_item(pattern, text) -> str:
